from typing import List
import subprocess
import threading
import time
import logging
import random
import math
from scipy.stats import poisson
from scipy.stats import expon
from concurrent import futures
import socket
from confluent_kafka import Producer
import grpc
import csle_collector.client_manager.client_manager_pb2_grpc
import csle_collector.client_manager.client_manager_pb2
import csle_collector.constants.constants as constants


class ClientThread(threading.Thread):
    """
    Thread representing a client
    """

    def __init__(self, service_time: float, commands: List[str], time_step_len_seconds: float) -> None:
        """
        Initializes the client

        :param service_time: the service time of the client
        """
        threading.Thread.__init__(self)
        self.service_time = service_time
        self.commands = commands
        self.time_step_len_seconds = time_step_len_seconds

    def run(self) -> None:
        """
        The main function of the client

        :return: None
        """
        start = time.time()
        done = False
        cmd_index = 0
        while not done:
            cmd = self.commands[cmd_index]
            p = subprocess.Popen(cmd, stdout=subprocess.PIPE, shell=True)
            p.communicate()
            p.wait()
            time.sleep(self.time_step_len_seconds)
            time_lapsed = time.time() - start
            if time_lapsed >= self.service_time:
                done = True
            else:
                if cmd_index < len(self.commands) - 1:
                    cmd_index += 1
                else:
                    cmd_index = 0


class ArrivalThread(threading.Thread):
    """
    Thread that generates client arrivals (starts client threads according to a Poisson process)
    """

    def __init__(self, commands: List[str], time_step_len_seconds: float = 1, lamb: float = 10, mu: float = 0.1,
                 num_commands: int = 2, sine_modulated: bool = False,
                 time_scaling_factor: float = 0.01, period_scaling_factor: float = 20):
        """
        Initializes the arrival thread

        :param time_step_len_seconds: the number of seconds that one time-unit of the Poisson process corresponds to
        :param lamb: the lambda parameter of the Poisson process for arrivals
        :param mu: the mu parameter of the service times of the clients
        :param commands: the list of commands that clients can use
        :param num_commands: the number of commands per client
        :param sine_modulated: boolean flag whether the rate of the arrival process is sine-modulated or not
        """
        threading.Thread.__init__(self)
        self.time_step_len_seconds = time_step_len_seconds
        self.client_threads = []
        self.t = 0
        self.lamb = lamb
        self.mu = mu
        self.stopped = False
        self.commands = commands
        self.num_commands = num_commands
        self.sine_modulated = sine_modulated
        self.rate = self.lamb
        self.time_scaling_factor = time_scaling_factor
        self.period_scaling_factor = period_scaling_factor
        logging.info(f"Starting arrival thread, lambda:{lamb}, mu:{mu}, num:commands:{num_commands}, "
                     f"commands:{commands}, sine_modulated: {sine_modulated}, "
                     f"time_scaling_factor: {time_scaling_factor}, period_scaling_factor: {period_scaling_factor}")

    def sine_modulated_poisson_rate(self, t) -> float:
        """
        Function that returns the rate of a sine-modulated Poisson process

        :param t: the time-step
        :return: the rate
        """
        return self.lamb + self.period_scaling_factor * math.sin(self.time_scaling_factor * math.pi * t)

    def run(self) -> None:
        """
        Runs the arrival generator, generates new clients dynamically according to a Poisson process

        :return: None
        """
        while not self.stopped:
            new_client_threads = []
            for ct in self.client_threads:
                if ct.is_alive():
                    new_client_threads.append(ct)
            self.client_threads = new_client_threads
            self.t += 1
            if self.sine_modulated:
                rate = self.sine_modulated_poisson_rate(self.t)
                self.rate = rate
                new_clients = poisson.rvs(rate, size=1)[0]
            else:
                new_clients = poisson.rvs(self.lamb, size=1)[0]
            for nc in range(new_clients):
                commands = random.sample(self.commands, self.num_commands)
                service_time = expon.rvs(scale=(self.mu * self.time_step_len_seconds), loc=0, size=1)[0]
                thread = ClientThread(service_time=service_time, commands=commands,
                                      time_step_len_seconds=self.time_step_len_seconds)
                thread.start()
                self.client_threads.append(thread)
            time.sleep(self.time_step_len_seconds)


class ProducerThread(threading.Thread):
    """
    Thread that pushes statistics to Kafka
    """

    def __init__(self, arrival_thread, time_step_len_seconds: int, ip: str, port: int):
        """
        Initializes the thread

        :param arrival_thread: the thread that manages the client arrivals, used to extract statistics
        :param time_step_len_seconds: the length between pushing statistics to Kafka
        :param ip: the ip of the Kafka server
        :param port: the port of the Kafka server
        """
        threading.Thread.__init__(self)
        self.arrival_thread = arrival_thread
        self.time_step_len_seconds = time_step_len_seconds
        self.stopped = False
        self.kafka_ip = ip
        self.port = port
        self.hostname = socket.gethostname()
        self.ip = socket.gethostbyname(self.hostname)
        self.conf = {
            constants.KAFKA.BOOTSTRAP_SERVERS_PROPERTY: f"{self.kafka_ip}:{self.port}",
            constants.KAFKA.CLIENT_ID_PROPERTY: self.hostname}
        self.producer = Producer(**self.conf)
        logging.info(f"Starting producer thread, ip:{self.ip}, kafka port:{self.port}, "
                     f"time_step_len:{self.time_step_len_seconds}, kafka_ip:{self.kafka_ip}")

    def run(self) -> None:
        """
        Main loop of the thread, pushes data to Kafka periodically

        :return: None
        """
        while not self.stopped and self.arrival_thread is not None:
            time.sleep(self.time_step_len_seconds)
            if self.arrival_thread is not None:
                ts = time.time()
                num_clients = len(self.arrival_thread.client_threads)
                rate = self.arrival_thread.rate
                self.producer.produce(constants.KAFKA_CONFIG.CLIENT_POPULATION_TOPIC_NAME,
                                      f"{ts},{self.ip},{num_clients},{rate}")
                self.producer.poll(0)


class ClientManagerServicer(csle_collector.client_manager.client_manager_pb2_grpc.ClientManagerServicer):
    """
    gRPC server for managing the running clients. Allows to start/stop clients remotely and also to query the
    state of the clients.
    """

    def __init__(self) -> None:
        """
        Initializes the server
        """
        self.arrival_thread = None
        self.producer_thread = None
        logging.basicConfig(filename=f"{constants.LOG_FILES.CLIENT_MANAGER_LOG_DIR}"
                                     f"{constants.LOG_FILES.CLIENT_MANAGER_LOG_FILE}", level=logging.INFO)

    def getClients(self, request: csle_collector.client_manager.client_manager_pb2.GetClientsMsg,
                   context: grpc.ServicerContext) -> csle_collector.client_manager.client_manager_pb2.ClientsDTO:
        """
        Gets the state of the clients

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a clients DTO with the state of the clients
        """
        num_clients = 0
        clients_time_step_len_seconds = 0
        producer_time_step_len_seconds = 0

        client_process_active = False
        if self.arrival_thread is not None:
            num_clients = len(self.arrival_thread.client_threads)
            client_process_active = True
            clients_time_step_len_seconds = self.arrival_thread.time_step_len_seconds

        producer_active = False
        if self.producer_thread is not None:
            producer_active = True
            producer_time_step_len_seconds = self.producer_thread.time_step_len_seconds

        clients_dto = csle_collector.client_manager.client_manager_pb2.ClientsDTO(
            num_clients=num_clients, client_process_active=client_process_active, producer_active=producer_active,
            clients_time_step_len_seconds=clients_time_step_len_seconds,
            producer_time_step_len_seconds=producer_time_step_len_seconds)
        return clients_dto

    def stopClients(self, request: csle_collector.client_manager.client_manager_pb2.StopClientsMsg,
                    context: grpc.ServicerContext):
        """
        Stops the Poisson-process that generates new clients

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a clients DTO with the state of the clients
        """
        logging.info("Stopping clients")

        clients_time_step_len_seconds = 0
        producer_time_step_len_seconds = 0

        if self.arrival_thread is not None:
            self.arrival_thread.stopped = True
            time.sleep(1)
        self.arrival_thread = None

        producer_active = False
        if self.producer_thread is not None:
            producer_active = True
            producer_time_step_len_seconds = self.producer_thread.time_step_len_seconds

        return csle_collector.client_manager.client_manager_pb2.ClientsDTO(
            num_clients=0, client_process_active=False, producer_active=producer_active,
            clients_time_step_len_seconds=clients_time_step_len_seconds,
            producer_time_step_len_seconds=producer_time_step_len_seconds)

    def startClients(self, request: csle_collector.client_manager.client_manager_pb2.StartClientsMsg,
                     context: grpc.ServicerContext) -> csle_collector.client_manager.client_manager_pb2.ClientsDTO:
        """
        Starts/Restarts the Poisson process that generates clients

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a clients DTO with the state of the clients
        """
        logging.info(f"Starting clients, commands:{request.commands}, lamb: {request.lamb}, mu: {request.mu}, "
                     f"sine_modulated: {request.sine_modulated}, "
                     f"time_scaling_factor: {request.time_scaling_factor}, "
                     f"period_scaling_factor: {request.period_scaling_factor}")

        producer_time_step_len_seconds = 0

        if self.arrival_thread is not None:
            self.arrival_thread.stopped = True
            time.sleep(1)

        if request.time_step_len_seconds <= 0:
            request.time_step_len_seconds = 1

        arrival_thread = ArrivalThread(commands=request.commands, time_step_len_seconds=request.time_step_len_seconds,
                                       lamb=request.lamb, mu=request.mu, sine_modulated=request.sine_modulated,
                                       time_scaling_factor=request.time_scaling_factor,
                                       period_scaling_factor=request.period_scaling_factor)
        arrival_thread.start()
        self.arrival_thread = arrival_thread
        clients_time_step_len_seconds = self.arrival_thread.time_step_len_seconds

        producer_active = False
        if self.producer_thread is not None:
            producer_active = True
            producer_time_step_len_seconds = self.producer_thread.time_step_len_seconds

        clients_dto = csle_collector.client_manager.client_manager_pb2.ClientsDTO(
            num_clients=len(self.arrival_thread.client_threads), client_process_active=True,
            producer_active=producer_active, clients_time_step_len_seconds=clients_time_step_len_seconds,
            producer_time_step_len_seconds=producer_time_step_len_seconds)
        return clients_dto

    def startProducer(self, request: csle_collector.client_manager.client_manager_pb2.StartProducerMsg,
                      context: grpc.ServicerContext) -> csle_collector.client_manager.client_manager_pb2.ClientsDTO:
        """
        Starts/Restarts the producer thread that pushes data to Kafka

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a clients DTO with the state of the clients
        """
        logging.info(f"Starting producer, time-step:{request.time_step_len_seconds}")

        clients_time_step_len_seconds = 0
        time.sleep(5)

        if self.producer_thread is not None:
            self.producer_thread.stopped = True
            time.sleep(1)

        if request.time_step_len_seconds <= 0:
            request.time_step_len_seconds = 1
        producer_thread = ProducerThread(arrival_thread=self.arrival_thread,
                                         time_step_len_seconds=request.time_step_len_seconds,
                                         ip=request.ip, port=request.port)
        producer_thread.start()
        self.producer_thread = producer_thread

        client_process_active = False
        num_clients = 0
        if self.arrival_thread is not None:
            num_clients = len(self.arrival_thread.client_threads)
            client_process_active = True
            clients_time_step_len_seconds = self.arrival_thread.time_step_len_seconds

        clients_dto = csle_collector.client_manager.client_manager_pb2.ClientsDTO(
            num_clients=num_clients, client_process_active=client_process_active, producer_active=True,
            clients_time_step_len_seconds=clients_time_step_len_seconds,
            producer_time_step_len_seconds=request.time_step_len_seconds)
        return clients_dto

    def stopProducer(self, request: csle_collector.client_manager.client_manager_pb2.StopProducerMsg,
                     context: grpc.ServicerContext) -> csle_collector.client_manager.client_manager_pb2.ClientsDTO:
        """
        Stops the producer thread that pushes data to Kafka

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a clients DTO with the state of the clients
        """
        logging.info("Stopping producer")

        clients_time_step_len_seconds = 0

        if self.producer_thread is not None:
            self.producer_thread.stopped = True
            time.sleep(1)
        self.producer_thread = None

        client_process_active = False
        num_clients = 0
        if self.arrival_thread is not None:
            num_clients = len(self.arrival_thread.client_threads)
            client_process_active = True
            clients_time_step_len_seconds = self.arrival_thread.time_step_len_seconds

        clients_dto = csle_collector.client_manager.client_manager_pb2.ClientsDTO(
            num_clients=num_clients, client_process_active=client_process_active, producer_active=False,
            clients_time_step_len_seconds=clients_time_step_len_seconds, producer_time_step_len_seconds=0)
        return clients_dto


def serve(port: int = 50044, log_dir: str = "/", log_file_name: str = "client_manager.log",
          max_workers: int = 10) -> None:
    """
    Starts the gRPC server for managing clients

    :param port: the port that the server will listen to
    :param log_dir: the directory to write the log file
    :param log_file_name: the file name of the log
    :param max_workers: the maximum number of GRPC workers
    :return: None
    """
    constants.LOG_FILES.CLIENT_MANAGER_LOG_DIR = log_dir
    constants.LOG_FILES.CLIENT_MANAGER_LOG_FILE = log_file_name
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=max_workers))
    csle_collector.client_manager.client_manager_pb2_grpc.add_ClientManagerServicer_to_server(
        ClientManagerServicer(), server)
    server.add_insecure_port(f'[::]:{port}')
    server.start()
    logging.info(f"ClientManager Server Started, Listening on port: {port}")
    server.wait_for_termination()


# Program entrypoint
if __name__ == '__main__':
    serve()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: host_manager.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12host_manager.proto\"\x14\n\x12StopHostMonitorMsg\"Z\n\x13StartHostMonitorMsg\x12\x10\n\x08kafka_ip\x18\x01 \x01(\t\x12\x12\n\nkafka_port\x18\x02 \x01(\x05\x12\x1d\n\x15time_step_len_seconds\x18\x04 \x01(\x05\"\x11\n\x0fStopFilebeatMsg\"\x12\n\x10StartFilebeatMsg\"\x97\x02\n\x11\x43onfigFilebeatMsg\x12\x11\n\tkibana_ip\x18\x01 \x01(\t\x12\x13\n\x0bkibana_port\x18\x02 \x01(\x05\x12\x12\n\nelastic_ip\x18\x03 \x01(\t\x12\x14\n\x0c\x65lastic_port\x18\x04 \x01(\x05\x12\x1a\n\x12num_elastic_shards\x18\x05 \x01(\x05\x12\x16\n\x0ereload_enabled\x18\x06 \x01(\x08\x12\x10\n\x08kafka_ip\x18\x07 \x01(\t\x12\x12\n\nkafka_port\x18\x08 \x01(\x05\x12\r\n\x05kafka\x18\t \x01(\x08\x12\x14\n\x0ckafka_topics\x18\n \x03(\t\x12\x18\n\x10\x66ilebeat_modules\x18\x0b \x03(\t\x12\x17\n\x0flog_files_paths\x18\x0c \x03(\t\"\x13\n\x11StopPacketbeatMsg\"\x14\n\x12StartPacketbeatMsg\"\x83\x01\n\x13\x43onfigPacketbeatMsg\x12\x11\n\tkibana_ip\x18\x01 \x01(\t\x12\x13\n\x0bkibana_port\x18\x02 \x01(\x05\x12\x12\n\nelastic_ip\x18\x03 \x01(\t\x12\x14\n\x0c\x65lastic_port\x18\x04 \x01(\x05\x12\x1a\n\x12num_elastic_shards\x18\x05 \x01(\x05\"\x13\n\x11StopMetricbeatMsg\"\x14\n\x12StartMetricbeatMsg\"\xdd\x01\n\x13\x43onfigMetricbeatMsg\x12\x11\n\tkibana_ip\x18\x01 \x01(\t\x12\x13\n\x0bkibana_port\x18\x02 \x01(\x05\x12\x12\n\nelastic_ip\x18\x03 \x01(\t\x12\x14\n\x0c\x65lastic_port\x18\x04 \x01(\x05\x12\x1a\n\x12num_elastic_shards\x18\x05 \x01(\x05\x12\x10\n\x08kafka_ip\x18\x06 \x01(\t\x12\x12\n\nkafka_port\x18\x07 \x01(\x05\x12\x16\n\x0ereload_enabled\x18\x08 \x01(\x08\x12\x1a\n\x12metricbeat_modules\x18\x0c \x03(\t\"\x12\n\x10GetHostStatusMsg\"\x95\x01\n\rHostStatusDTO\x12\x17\n\x0fmonitor_running\x18\x01 \x01(\x08\x12\x18\n\x10\x66ilebeat_running\x18\x02 \x01(\x08\x12\x1a\n\x12packetbeat_running\x18\x03 \x01(\x08\x12\x1a\n\x12metricbeat_running\x18\x04 \x01(\x08\x12\x19\n\x11heartbeat_running\x18\x05 \x01(\x08\"G\n\x11GetHostMetricsMsg\x12\x1b\n\x13\x66\x61iled_auth_last_ts\x18\x01 \x01(\x02\x12\x15\n\rlogin_last_ts\x18\x02 \x01(\x02\"\xd1\x01\n\x0eHostMetricsDTO\x12\x1b\n\x13num_logged_in_users\x18\x01 \x01(\x05\x12!\n\x19num_failed_login_attempts\x18\x02 \x01(\x05\x12\x1c\n\x14num_open_connections\x18\x03 \x01(\x05\x12\x18\n\x10num_login_events\x18\x04 \x01(\x05\x12\x15\n\rnum_processes\x18\x05 \x01(\x05\x12\x11\n\tnum_users\x18\x06 \x01(\x05\x12\n\n\x02ip\x18\x07 \x01(\t\x12\x11\n\ttimestamp\x18\x08 \x01(\x02\"\x12\n\x10StopHeartbeatMsg\"\x13\n\x11StartHeartbeatMsg\"\x9c\x01\n\x12\x43onfigHeartbeatMsg\x12\x11\n\tkibana_ip\x18\x01 \x01(\t\x12\x13\n\x0bkibana_port\x18\x02 \x01(\x05\x12\x12\n\nelastic_ip\x18\x03 \x01(\t\x12\x14\n\x0c\x65lastic_port\x18\x04 \x01(\x05\x12\x1a\n\x12num_elastic_shards\x18\x05 \x01(\x05\x12\x18\n\x10hosts_to_monitor\x18\x06 \x03(\t2\x98\x07\n\x0bHostManager\x12\x38\n\x0fstopHostMonitor\x12\x13.StopHostMonitorMsg\x1a\x0e.HostStatusDTO\"\x00\x12:\n\x10startHostMonitor\x12\x14.StartHostMonitorMsg\x1a\x0e.HostStatusDTO\"\x00\x12\x34\n\rgetHostStatus\x12\x11.GetHostStatusMsg\x1a\x0e.HostStatusDTO\"\x00\x12\x37\n\x0egetHostMetrics\x12\x12.GetHostMetricsMsg\x1a\x0f.HostMetricsDTO\"\x00\x12\x32\n\x0cstopFilebeat\x12\x10.StopFilebeatMsg\x1a\x0e.HostStatusDTO\"\x00\x12\x34\n\rstartFilebeat\x12\x11.StartFilebeatMsg\x1a\x0e.HostStatusDTO\"\x00\x12\x36\n\x0e\x63onfigFilebeat\x12\x12.ConfigFilebeatMsg\x1a\x0e.HostStatusDTO\"\x00\x12\x36\n\x0estopPacketbeat\x12\x12.StopPacketbeatMsg\x1a\x0e.HostStatusDTO\"\x00\x12\x38\n\x0fstartPacketbeat\x12\x13.StartPacketbeatMsg\x1a\x0e.HostStatusDTO\"\x00\x12:\n\x10\x63onfigPacketbeat\x12\x14.ConfigPacketbeatMsg\x1a\x0e.HostStatusDTO\"\x00\x12\x36\n\x0estopMetricbeat\x12\x12.StopMetricbeatMsg\x1a\x0e.HostStatusDTO\"\x00\x12\x38\n\x0fstartMetricbeat\x12\x13.StartMetricbeatMsg\x1a\x0e.HostStatusDTO\"\x00\x12:\n\x10\x63onfigMetricbeat\x12\x14.ConfigMetricbeatMsg\x1a\x0e.HostStatusDTO\"\x00\x12\x34\n\rstopHeartbeat\x12\x11.StopHeartbeatMsg\x1a\x0e.HostStatusDTO\"\x00\x12\x36\n\x0estartHeartbeat\x12\x12.StartHeartbeatMsg\x1a\x0e.HostStatusDTO\"\x00\x12\x38\n\x0f\x63onfigHeartbeat\x12\x13.ConfigHeartbeatMsg\x1a\x0e.HostStatusDTO\"\x00\x62\x06proto3')



_STOPHOSTMONITORMSG = DESCRIPTOR.message_types_by_name['StopHostMonitorMsg']
_STARTHOSTMONITORMSG = DESCRIPTOR.message_types_by_name['StartHostMonitorMsg']
_STOPFILEBEATMSG = DESCRIPTOR.message_types_by_name['StopFilebeatMsg']
_STARTFILEBEATMSG = DESCRIPTOR.message_types_by_name['StartFilebeatMsg']
_CONFIGFILEBEATMSG = DESCRIPTOR.message_types_by_name['ConfigFilebeatMsg']
_STOPPACKETBEATMSG = DESCRIPTOR.message_types_by_name['StopPacketbeatMsg']
_STARTPACKETBEATMSG = DESCRIPTOR.message_types_by_name['StartPacketbeatMsg']
_CONFIGPACKETBEATMSG = DESCRIPTOR.message_types_by_name['ConfigPacketbeatMsg']
_STOPMETRICBEATMSG = DESCRIPTOR.message_types_by_name['StopMetricbeatMsg']
_STARTMETRICBEATMSG = DESCRIPTOR.message_types_by_name['StartMetricbeatMsg']
_CONFIGMETRICBEATMSG = DESCRIPTOR.message_types_by_name['ConfigMetricbeatMsg']
_GETHOSTSTATUSMSG = DESCRIPTOR.message_types_by_name['GetHostStatusMsg']
_HOSTSTATUSDTO = DESCRIPTOR.message_types_by_name['HostStatusDTO']
_GETHOSTMETRICSMSG = DESCRIPTOR.message_types_by_name['GetHostMetricsMsg']
_HOSTMETRICSDTO = DESCRIPTOR.message_types_by_name['HostMetricsDTO']
_STOPHEARTBEATMSG = DESCRIPTOR.message_types_by_name['StopHeartbeatMsg']
_STARTHEARTBEATMSG = DESCRIPTOR.message_types_by_name['StartHeartbeatMsg']
_CONFIGHEARTBEATMSG = DESCRIPTOR.message_types_by_name['ConfigHeartbeatMsg']
StopHostMonitorMsg = _reflection.GeneratedProtocolMessageType('StopHostMonitorMsg', (_message.Message,), {
  'DESCRIPTOR' : _STOPHOSTMONITORMSG,
  '__module__' : 'host_manager_pb2'
  # @@protoc_insertion_point(class_scope:StopHostMonitorMsg)
  })
_sym_db.RegisterMessage(StopHostMonitorMsg)

StartHostMonitorMsg = _reflection.GeneratedProtocolMessageType('StartHostMonitorMsg', (_message.Message,), {
  'DESCRIPTOR' : _STARTHOSTMONITORMSG,
  '__module__' : 'host_manager_pb2'
  # @@protoc_insertion_point(class_scope:StartHostMonitorMsg)
  })
_sym_db.RegisterMessage(StartHostMonitorMsg)

StopFilebeatMsg = _reflection.GeneratedProtocolMessageType('StopFilebeatMsg', (_message.Message,), {
  'DESCRIPTOR' : _STOPFILEBEATMSG,
  '__module__' : 'host_manager_pb2'
  # @@protoc_insertion_point(class_scope:StopFilebeatMsg)
  })
_sym_db.RegisterMessage(StopFilebeatMsg)

StartFilebeatMsg = _reflection.GeneratedProtocolMessageType('StartFilebeatMsg', (_message.Message,), {
  'DESCRIPTOR' : _STARTFILEBEATMSG,
  '__module__' : 'host_manager_pb2'
  # @@protoc_insertion_point(class_scope:StartFilebeatMsg)
  })
_sym_db.RegisterMessage(StartFilebeatMsg)

ConfigFilebeatMsg = _reflection.GeneratedProtocolMessageType('ConfigFilebeatMsg', (_message.Message,), {
  'DESCRIPTOR' : _CONFIGFILEBEATMSG,
  '__module__' : 'host_manager_pb2'
  # @@protoc_insertion_point(class_scope:ConfigFilebeatMsg)
  })
_sym_db.RegisterMessage(ConfigFilebeatMsg)

StopPacketbeatMsg = _reflection.GeneratedProtocolMessageType('StopPacketbeatMsg', (_message.Message,), {
  'DESCRIPTOR' : _STOPPACKETBEATMSG,
  '__module__' : 'host_manager_pb2'
  # @@protoc_insertion_point(class_scope:StopPacketbeatMsg)
  })
_sym_db.RegisterMessage(StopPacketbeatMsg)

StartPacketbeatMsg = _reflection.GeneratedProtocolMessageType('StartPacketbeatMsg', (_message.Message,), {
  'DESCRIPTOR' : _STARTPACKETBEATMSG,
  '__module__' : 'host_manager_pb2'
  # @@protoc_insertion_point(class_scope:StartPacketbeatMsg)
  })
_sym_db.RegisterMessage(StartPacketbeatMsg)

ConfigPacketbeatMsg = _reflection.GeneratedProtocolMessageType('ConfigPacketbeatMsg', (_message.Message,), {
  'DESCRIPTOR' : _CONFIGPACKETBEATMSG,
  '__module__' : 'host_manager_pb2'
  # @@protoc_insertion_point(class_scope:ConfigPacketbeatMsg)
  })
_sym_db.RegisterMessage(ConfigPacketbeatMsg)

StopMetricbeatMsg = _reflection.GeneratedProtocolMessageType('StopMetricbeatMsg', (_message.Message,), {
  'DESCRIPTOR' : _STOPMETRICBEATMSG,
  '__module__' : 'host_manager_pb2'
  # @@protoc_insertion_point(class_scope:StopMetricbeatMsg)
  })
_sym_db.RegisterMessage(StopMetricbeatMsg)

StartMetricbeatMsg = _reflection.GeneratedProtocolMessageType('StartMetricbeatMsg', (_message.Message,), {
  'DESCRIPTOR' : _STARTMETRICBEATMSG,
  '__module__' : 'host_manager_pb2'
  # @@protoc_insertion_point(class_scope:StartMetricbeatMsg)
  })
_sym_db.RegisterMessage(StartMetricbeatMsg)

ConfigMetricbeatMsg = _reflection.GeneratedProtocolMessageType('ConfigMetricbeatMsg', (_message.Message,), {
  'DESCRIPTOR' : _CONFIGMETRICBEATMSG,
  '__module__' : 'host_manager_pb2'
  # @@protoc_insertion_point(class_scope:ConfigMetricbeatMsg)
  })
_sym_db.RegisterMessage(ConfigMetricbeatMsg)

GetHostStatusMsg = _reflection.GeneratedProtocolMessageType('GetHostStatusMsg', (_message.Message,), {
  'DESCRIPTOR' : _GETHOSTSTATUSMSG,
  '__module__' : 'host_manager_pb2'
  # @@protoc_insertion_point(class_scope:GetHostStatusMsg)
  })
_sym_db.RegisterMessage(GetHostStatusMsg)

HostStatusDTO = _reflection.GeneratedProtocolMessageType('HostStatusDTO', (_message.Message,), {
  'DESCRIPTOR' : _HOSTSTATUSDTO,
  '__module__' : 'host_manager_pb2'
  # @@protoc_insertion_point(class_scope:HostStatusDTO)
  })
_sym_db.RegisterMessage(HostStatusDTO)

GetHostMetricsMsg = _reflection.GeneratedProtocolMessageType('GetHostMetricsMsg', (_message.Message,), {
  'DESCRIPTOR' : _GETHOSTMETRICSMSG,
  '__module__' : 'host_manager_pb2'
  # @@protoc_insertion_point(class_scope:GetHostMetricsMsg)
  })
_sym_db.RegisterMessage(GetHostMetricsMsg)

HostMetricsDTO = _reflection.GeneratedProtocolMessageType('HostMetricsDTO', (_message.Message,), {
  'DESCRIPTOR' : _HOSTMETRICSDTO,
  '__module__' : 'host_manager_pb2'
  # @@protoc_insertion_point(class_scope:HostMetricsDTO)
  })
_sym_db.RegisterMessage(HostMetricsDTO)

StopHeartbeatMsg = _reflection.GeneratedProtocolMessageType('StopHeartbeatMsg', (_message.Message,), {
  'DESCRIPTOR' : _STOPHEARTBEATMSG,
  '__module__' : 'host_manager_pb2'
  # @@protoc_insertion_point(class_scope:StopHeartbeatMsg)
  })
_sym_db.RegisterMessage(StopHeartbeatMsg)

StartHeartbeatMsg = _reflection.GeneratedProtocolMessageType('StartHeartbeatMsg', (_message.Message,), {
  'DESCRIPTOR' : _STARTHEARTBEATMSG,
  '__module__' : 'host_manager_pb2'
  # @@protoc_insertion_point(class_scope:StartHeartbeatMsg)
  })
_sym_db.RegisterMessage(StartHeartbeatMsg)

ConfigHeartbeatMsg = _reflection.GeneratedProtocolMessageType('ConfigHeartbeatMsg', (_message.Message,), {
  'DESCRIPTOR' : _CONFIGHEARTBEATMSG,
  '__module__' : 'host_manager_pb2'
  # @@protoc_insertion_point(class_scope:ConfigHeartbeatMsg)
  })
_sym_db.RegisterMessage(ConfigHeartbeatMsg)

_HOSTMANAGER = DESCRIPTOR.services_by_name['HostManager']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _STOPHOSTMONITORMSG._serialized_start=22
  _STOPHOSTMONITORMSG._serialized_end=42
  _STARTHOSTMONITORMSG._serialized_start=44
  _STARTHOSTMONITORMSG._serialized_end=134
  _STOPFILEBEATMSG._serialized_start=136
  _STOPFILEBEATMSG._serialized_end=153
  _STARTFILEBEATMSG._serialized_start=155
  _STARTFILEBEATMSG._serialized_end=173
  _CONFIGFILEBEATMSG._serialized_start=176
  _CONFIGFILEBEATMSG._serialized_end=455
  _STOPPACKETBEATMSG._serialized_start=457
  _STOPPACKETBEATMSG._serialized_end=476
  _STARTPACKETBEATMSG._serialized_start=478
  _STARTPACKETBEATMSG._serialized_end=498
  _CONFIGPACKETBEATMSG._serialized_start=501
  _CONFIGPACKETBEATMSG._serialized_end=632
  _STOPMETRICBEATMSG._serialized_start=634
  _STOPMETRICBEATMSG._serialized_end=653
  _STARTMETRICBEATMSG._serialized_start=655
  _STARTMETRICBEATMSG._serialized_end=675
  _CONFIGMETRICBEATMSG._serialized_start=678
  _CONFIGMETRICBEATMSG._serialized_end=899
  _GETHOSTSTATUSMSG._serialized_start=901
  _GETHOSTSTATUSMSG._serialized_end=919
  _HOSTSTATUSDTO._serialized_start=922
  _HOSTSTATUSDTO._serialized_end=1071
  _GETHOSTMETRICSMSG._serialized_start=1073
  _GETHOSTMETRICSMSG._serialized_end=1144
  _HOSTMETRICSDTO._serialized_start=1147
  _HOSTMETRICSDTO._serialized_end=1356
  _STOPHEARTBEATMSG._serialized_start=1358
  _STOPHEARTBEATMSG._serialized_end=1376
  _STARTHEARTBEATMSG._serialized_start=1378
  _STARTHEARTBEATMSG._serialized_end=1397
  _CONFIGHEARTBEATMSG._serialized_start=1400
  _CONFIGHEARTBEATMSG._serialized_end=1556
  _HOSTMANAGER._serialized_start=1559
  _HOSTMANAGER._serialized_end=2479
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: client_manager.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x14\x63lient_manager.proto\"\x10\n\x0eStopClientsMsg\"\xc6\x01\n\x0fStartClientsMsg\x12\n\n\x02mu\x18\x01 \x01(\x02\x12\x0c\n\x04lamb\x18\x02 \x01(\x02\x12\x1d\n\x15time_step_len_seconds\x18\x03 \x01(\x05\x12\x10\n\x08\x63ommands\x18\x04 \x03(\t\x12\x14\n\x0cnum_commands\x18\x05 \x01(\x05\x12\x16\n\x0esine_modulated\x18\x06 \x01(\x08\x12\x1b\n\x13time_scaling_factor\x18\x07 \x01(\x02\x12\x1d\n\x15period_scaling_factor\x18\x08 \x01(\x02\"\x0f\n\rGetClientsMsg\"K\n\x10StartProducerMsg\x12\n\n\x02ip\x18\x01 \x01(\t\x12\x0c\n\x04port\x18\x02 \x01(\x05\x12\x1d\n\x15time_step_len_seconds\x18\x03 \x01(\x05\"\x11\n\x0fStopProducerMsg\"\xa8\x01\n\nClientsDTO\x12\x13\n\x0bnum_clients\x18\x01 \x01(\x05\x12\x1d\n\x15\x63lient_process_active\x18\x02 \x01(\x08\x12\x17\n\x0fproducer_active\x18\x03 \x01(\x08\x12%\n\x1d\x63lients_time_step_len_seconds\x18\x04 \x01(\x05\x12&\n\x1eproducer_time_step_len_seconds\x18\x05 \x01(\x05\x32\x80\x02\n\rClientManager\x12+\n\ngetClients\x12\x0e.GetClientsMsg\x1a\x0b.ClientsDTO\"\x00\x12-\n\x0bstopClients\x12\x0f.StopClientsMsg\x1a\x0b.ClientsDTO\"\x00\x12/\n\x0cstartClients\x12\x10.StartClientsMsg\x1a\x0b.ClientsDTO\"\x00\x12\x31\n\rstartProducer\x12\x11.StartProducerMsg\x1a\x0b.ClientsDTO\"\x00\x12/\n\x0cstopProducer\x12\x10.StopProducerMsg\x1a\x0b.ClientsDTO\"\x00\x62\x06proto3')



_STOPCLIENTSMSG = DESCRIPTOR.message_types_by_name['StopClientsMsg']
_STARTCLIENTSMSG = DESCRIPTOR.message_types_by_name['StartClientsMsg']
_GETCLIENTSMSG = DESCRIPTOR.message_types_by_name['GetClientsMsg']
_STARTPRODUCERMSG = DESCRIPTOR.message_types_by_name['StartProducerMsg']
_STOPPRODUCERMSG = DESCRIPTOR.message_types_by_name['StopProducerMsg']
_CLIENTSDTO = DESCRIPTOR.message_types_by_name['ClientsDTO']
StopClientsMsg = _reflection.GeneratedProtocolMessageType('StopClientsMsg', (_message.Message,), {
  'DESCRIPTOR' : _STOPCLIENTSMSG,
  '__module__' : 'client_manager_pb2'
  # @@protoc_insertion_point(class_scope:StopClientsMsg)
  })
_sym_db.RegisterMessage(StopClientsMsg)

StartClientsMsg = _reflection.GeneratedProtocolMessageType('StartClientsMsg', (_message.Message,), {
  'DESCRIPTOR' : _STARTCLIENTSMSG,
  '__module__' : 'client_manager_pb2'
  # @@protoc_insertion_point(class_scope:StartClientsMsg)
  })
_sym_db.RegisterMessage(StartClientsMsg)

GetClientsMsg = _reflection.GeneratedProtocolMessageType('GetClientsMsg', (_message.Message,), {
  'DESCRIPTOR' : _GETCLIENTSMSG,
  '__module__' : 'client_manager_pb2'
  # @@protoc_insertion_point(class_scope:GetClientsMsg)
  })
_sym_db.RegisterMessage(GetClientsMsg)

StartProducerMsg = _reflection.GeneratedProtocolMessageType('StartProducerMsg', (_message.Message,), {
  'DESCRIPTOR' : _STARTPRODUCERMSG,
  '__module__' : 'client_manager_pb2'
  # @@protoc_insertion_point(class_scope:StartProducerMsg)
  })
_sym_db.RegisterMessage(StartProducerMsg)

StopProducerMsg = _reflection.GeneratedProtocolMessageType('StopProducerMsg', (_message.Message,), {
  'DESCRIPTOR' : _STOPPRODUCERMSG,
  '__module__' : 'client_manager_pb2'
  # @@protoc_insertion_point(class_scope:StopProducerMsg)
  })
_sym_db.RegisterMessage(StopProducerMsg)

ClientsDTO = _reflection.GeneratedProtocolMessageType('ClientsDTO', (_message.Message,), {
  'DESCRIPTOR' : _CLIENTSDTO,
  '__module__' : 'client_manager_pb2'
  # @@protoc_insertion_point(class_scope:ClientsDTO)
  })
_sym_db.RegisterMessage(ClientsDTO)

_CLIENTMANAGER = DESCRIPTOR.services_by_name['ClientManager']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _STOPCLIENTSMSG._serialized_start=24
  _STOPCLIENTSMSG._serialized_end=40
  _STARTCLIENTSMSG._serialized_start=43
  _STARTCLIENTSMSG._serialized_end=241
  _GETCLIENTSMSG._serialized_start=243
  _GETCLIENTSMSG._serialized_end=258
  _STARTPRODUCERMSG._serialized_start=260
  _STARTPRODUCERMSG._serialized_end=335
  _STOPPRODUCERMSG._serialized_start=337
  _STOPPRODUCERMSG._serialized_end=354
  _CLIENTSDTO._serialized_start=357
  _CLIENTSDTO._serialized_end=525
  _CLIENTMANAGER._serialized_start=528
  _CLIENTMANAGER._serialized_end=784
# @@protoc_insertion_point(module_scope)



from typing import List
import csle_collector.client_manager.client_manager_pb2_grpc
import csle_collector.client_manager.client_manager_pb2
import csle_collector.constants.constants as constants


def get_clients(stub: csle_collector.client_manager.client_manager_pb2_grpc.ClientManagerStub,
                timeout=constants.GRPC.TIMEOUT_SECONDS) \
        -> csle_collector.client_manager.client_manager_pb2.ClientsDTO:
    """
    Queries the server for the client state

    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :return: a clientsDTO describing the state of the clients
    """
    get_clients_dto_msg = csle_collector.client_manager.client_manager_pb2.GetClientsMsg()
    clients_dto = stub.getClients(get_clients_dto_msg)
    return clients_dto


def stop_clients(stub: csle_collector.client_manager.client_manager_pb2_grpc.ClientManagerStub,
                 timeout=constants.GRPC.TIMEOUT_SECONDS):
    """
    Stops the client arrival process

    :param stub: the stub to the gRPC server
    :param timeout: the GRPC timeout (seconds)
    :return: a clientsDTO describing the state of the clients
    """
    stop_clients_msg = csle_collector.client_manager.client_manager_pb2.StopClientsMsg()
    clients_dto = stub.stopClients(stop_clients_msg)
    return clients_dto


def start_clients(stub: csle_collector.client_manager.client_manager_pb2_grpc.ClientManagerStub,
                  mu: float, lamb: float, time_step_len_seconds: int, commands: List[str], num_commands: int = 2,
                  sine_modulated: bool = False, time_scaling_factor: float = 0.01, period_scaling_factor: float = 20,
                  timeout=constants.GRPC.TIMEOUT_SECONDS):
    """
    Starts the client arrival process

    :param stub: the stub to the gRPC server
    :param mu: the mu parameter for the Exponential service time
    :param lamb: the lambda parameter for the Poisson process
    :param time_step_len_seconds: the length of a time-step for simulating the arrival process
    :param num:commands: the number of commands that each client will use
    :param sine_modulated: whether the arrival process is sine modulated or not
    :param time_scaling_factor: the time scaling factor for the sine modulated arrival process
    :param period_scaling_factor: the period scaling factor for the sine modulated arrival process
    :param timeout: the GRPC timeout (seconds)
    :return: a clientsDTO describing the state of the clients
    """
    start_clients_msg = csle_collector.client_manager.client_manager_pb2.StartClientsMsg(
        mu=mu, lamb=lamb, time_step_len_seconds=time_step_len_seconds, commands=commands,
        num_commands=num_commands, sine_modulated=sine_modulated, period_scaling_factor=period_scaling_factor,
        time_scaling_factor=time_scaling_factor
    )
    clients_dto = stub.startClients(start_clients_msg)
    return clients_dto


def stop_producer(stub: csle_collector.client_manager.client_manager_pb2_grpc.ClientManagerStub,
                  timeout=constants.GRPC.TIMEOUT_SECONDS):
    """
    Stops the producer process

    :param stub: the stub to the gRPC server
    :param timeout: the GRPC timeout (seconds)
    :return: a clientsDTO describing the state of the clients
    """
    stop_producer_msg = csle_collector.client_manager.client_manager_pb2.StopProducerMsg()
    clients_dto = stub.stopProducer(stop_producer_msg, timeout=timeout)
    return clients_dto


def start_producer(stub: csle_collector.client_manager.client_manager_pb2_grpc.ClientManagerStub, ip: str, port: int,
                   time_step_len_seconds: int, timeout=constants.GRPC.TIMEOUT_SECONDS):
    """
    Stops the producer process

    :param stub: the stub to the gRPC server
    :param ip: ip of the kafka server to produce to
    :param port: port of the kafka server to produce to
    :param timeout: the GRPC timeout (seconds)
    :return: a clientsDTO describing the state of the clients
    """
    start_producer_msg = csle_collector.client_manager.client_manager_pb2.StartProducerMsg(
        ip=ip,
        port=port,
        time_step_len_seconds=time_step_len_seconds
    )
    clients_dto = stub.startProducer(start_producer_msg, timeout=timeout)
    return clients_dto
